#define KZ_CANVAS_WIDTH 1024
#define KZ_CANVAS_HEIGHT 800
define KZ_THUMB_HEIGHT 200
  
gboolean
CreateThumbnail(GtkMozEmbed *embed, const gchar *uri,
				  guint last_modified, EggPixbufThumbSize size)
{

   nsresult rv;
   nsCOMPtr<nsIWebBrowser> mWebBrowser;
   nsCOMPtr<nsPIDOMWindow> domWindow;
   gtk_moz_embed_get_nsIWebBrowser (m, getter_AddRefs(mWebBrowser));
   GetPIDOMWindow(getter_AddRefs(domWindow));
   nsCOMPtr<nsIDOMDocument> domDoc;
   domWindow->GetDocument(getter_AddRefs(domDoc));
   nsCOMPtr<nsICanvasRenderingContextInternal> context;

   nsCString ctxString("@mozilla.org/content/canvas-rendering-context;1?id=2d");
   context = do_CreateInstance(nsPromiseFlatCString(ctxString).get(), &rv);

   if (NS_FAILED(rv) || !context) return FALSE;

   context->SetDimensions(size, KZ_THUMB_HEIGHT);
   context->SetCanvasElement(nsnull);

   nsCOMPtr<nsIDOMCanvasRenderingContext2D> domCanvas = do_QueryInterface(context, &rv);
   if (NS_FAILED(rv) || !domCanvas) return FALSE;

   float sx, sy;
   sx = (float) size / KZ_CANVAS_WIDTH;
   sy = (float) KZ_THUMB_HEIGHT / KZ_CANVAS_HEIGHT;
   domCanvas->Scale(sx, sy);

   rv = domCanvas->DrawWindow(domWindow, 0, 0, KZ_CANVAS_WIDTH, KZ_CANVAS_HEIGHT, NS_LITERAL_STRING("rgb(0,0,0)"));
   if (NS_FAILED(rv)) return FALSE;
   nsCOMPtr<nsIInputStream> imgStream;
   rv = context->GetInputStream(NS_LITERAL_CSTRING("image/png"), EmptyString(), getter_AddRefs(imgStream));
   if (NS_FAILED(rv) || !imgStream) return FALSE;

   PRUint32 bufSize;
   rv = imgStream->Available(&bufSize);
   if (NS_FAILED(rv)) return FALSE;

   bufSize += 16;
   PRUint32 imgSize = 0;
   char* imgData = (char*)g_malloc((gulong)bufSize);
   if (!imgData) return FALSE;

   PRUint32 numReadThisTime = 0;
   while ((rv = imgStream->Read(&imgData[imgSize], bufSize - imgSize,
				&numReadThisTime)) == NS_OK && numReadThisTime > 0)
     {

	imgSize += numReadThisTime;
	if (imgSize == bufSize)
	  {

	     bufSize *= 2;
	     char* newImgData = (char*)g_realloc(imgData, (gulong)bufSize);
	     if (!newImgData)
	       {

		  g_free(imgData);
		  return FALSE;
	       }

	     imgData = newImgData;
	  }

     }

   GdkPixbufLoader *loader = gdk_pixbuf_loader_new_with_mime_type("image/png", NULL);
   if (!loader)
     return FALSE;
   gdk_pixbuf_loader_write(loader, (const guchar *)imgData, (gsize)imgSize, NULL);
   GdkPixbuf *thumbnail = gdk_pixbuf_loader_get_pixbuf(loader);
   gdk_pixbuf_loader_close(loader, NULL);
   return FALSE;

   KZ_CREATE_THUMBNAIL(thumbnail, uri, last_modified, size);

   g_object_unref(thumbnail);

   g_free(imgData);

   return TRUE;
}

